@@ -1,11 +1,7 @@
-# -*- coding: utf-8 -*-
-import marshal
 import os
 import unittest
 from logging import getLogger
-
-from six import PY2
-from six.moves.urllib.parse import quote
+from urllib.parse import quote
 
 from AccessControl.owner import EmergencyUserCannotOwn
 from AccessControl.SecurityManagement import newSecurityManager
@@ -14,16 +10,14 @@ from AccessControl.SecurityManager import setSecurityPolicy
 from AccessControl.SpecialUsers import emergency_user
 from AccessControl.SpecialUsers import nobody
 from AccessControl.SpecialUsers import system
-from AccessControl.User import User  # before SpecialUsers
+from AccessControl.users import User  # before SpecialUsers
 from Acquisition import Implicit
 from Acquisition import aq_self
 from App.config import getConfiguration
-from OFS import bbb
 from OFS.interfaces import IItem
 from OFS.metaconfigure import setDeprecatedManageAddDelete
 from OFS.ObjectManager import ObjectManager
 from OFS.SimpleItem import SimpleItem
-from Testing.makerequest import makerequest
 from zExceptions import BadRequest
 from Zope2.App import zcml
 from zope.component.testing import PlacelessSetup
@@ -87,7 +81,7 @@ class ObjectManagerWithIItem(ObjectManager):
 class ObjectManagerTests(PlacelessSetup, unittest.TestCase):
 
     def setUp(self):
-        super(ObjectManagerTests, self).setUp()
+        super().setUp()
         self.saved_cfg_debug_mode = getConfiguration().debug_mode
         import Zope2.App
         zcml.load_config('meta.zcml', Zope2.App)
@@ -98,7 +92,7 @@ class ObjectManagerTests(PlacelessSetup, unittest.TestCase):
     def tearDown(self):
         noSecurityManager()
         getConfiguration().debug_mode = self.saved_cfg_debug_mode
-        super(ObjectManagerTests, self).tearDown()
+        super().tearDown()
 
     def setDebugMode(self, mode):
         getConfiguration().debug_mode = mode
@@ -120,7 +114,7 @@ class ObjectManagerTests(PlacelessSetup, unittest.TestCase):
 
     def test_filtered_meta_types(self):
 
-        class _DummySecurityPolicy(object):
+        class _DummySecurityPolicy:
 
             def checkPermission(self, permission, object, context):
                 return permission == 'addFoo'
@@ -341,7 +335,7 @@ class ObjectManagerTests(PlacelessSetup, unittest.TestCase):
         self.assertFalse('stuff' in om)
 
         om._setObject('stuff', ob)
-        om.manage_delObjects(u'stuff')
+        om.manage_delObjects('stuff')
         self.assertFalse('stuff' in om)
 
     def test_hasObject(self):
@@ -439,10 +433,7 @@ class ObjectManagerTests(PlacelessSetup, unittest.TestCase):
         om['2'] = si2
         iterator = iter(om)
         self.assertTrue(hasattr(iterator, '__iter__'))
-        if PY2:
-            self.assertTrue(hasattr(iterator, 'next'))
-        else:
-            self.assertTrue(hasattr(iterator, '__next__'))
+        self.assertTrue(hasattr(iterator, '__next__'))
         result = [i for i in iterator]
         self.assertTrue('1' in result)
         self.assertTrue('2' in result)
@@ -552,33 +543,83 @@ class ObjectManagerTests(PlacelessSetup, unittest.TestCase):
         # Cleanup
         getConfiguration().zmi_bookmarkable_urls = saved_state
 
-    @unittest.skipUnless(bbb.HAS_ZSERVER, 'Test requires ZServer')
-    def test_FTPList(self):
+    def test_findChildren(self):
         from OFS.Folder import Folder
-        om = makerequest(self._makeOne())
-        om.isTopLevelPrincipiaApplicationObject = True
-        om._setObject('sub', Folder('sub'))
-        om.sub._setObject('subsub', Folder('subsub'))
-        req = om.REQUEST
-        req.PARENTS = [om]
+        from OFS.Image import File
+        from OFS.ObjectManager import findChildren
+        top = Folder("top")
+        f1 = File("f1", "", b"")
+        top._setObject(f1.getId(), f1)
+        fo = Folder("fo")
+        top._setObject(fo.getId(), fo)
+        f2 = File("f2", "", b"")
+        fo._setObject(f2.getId(), f2)
+        self.assertEqual(
+            [ci[0] for ci in findChildren(top)],
+            ["top/f1",
+             # surprisingly, `findChildren` ignores folderish children
+             # "top/fo",
+             "top/fo/f2"])
 
-        # At the root we only see a single entry for the subfolder
-        data = marshal.loads(om.manage_FTPlist(req))
-        self.assertEqual(len(data), 1)
-        self.assertEqual(data[0][0], 'sub')
+    def test_export_import(self):
+        import tempfile
+        from os import mkdir
+        from os import rmdir
+        from os import unlink
+        from os.path import join
+        from os.path import split
 
-        # In the subfolder, we see an entry for the current folder
-        # and the folder in it
-        data = marshal.loads(om.sub.manage_FTPlist(req))
-        self.assertEqual(len(data), 2)
-        self.assertEqual(data[0][0], '.')
-        self.assertEqual(data[1][0], 'subsub')
+        from OFS.Folder import Folder
+        from OFS.Image import File
+        from transaction import commit
+        from ZODB.DB import DB
+        from ZODB.DemoStorage import DemoStorage
+        try:
+            tf = None  # temporary file required for export/import
+            # export/import needs the object manager in ZODB
+            s = DemoStorage()
+            db = DB(s)
+            c = db.open()
+            root = c.root()
+            top = Folder("top")
+            f = File("f", "", b"")
+            top._setObject(f.getId(), f)
+            root["top"] = top
+            tmp = Folder("tmp")
+            top._setObject(tmp.getId(), tmp)
+            commit()
+            exported = top.manage_exportObject("f", True)
+            tdir = tempfile.mkdtemp()
+            idir = join(tdir, "import")
+            mkdir(idir)
+            tf = tempfile.NamedTemporaryFile(
+                dir=idir, delete=False)
+            tf.write(exported)
+            tf.close()
+            unused, tname = split(tf.name)
+            tmp._getImportPaths = _CallResult((tdir,))
+            tmp.manage_importObject(tname, set_owner=False,
+                                    suppress_events=True)
+            imp_f = tmp["f"]  # exception if import unsuccessful
+            self.assertIsInstance(imp_f, File)
+            commit()
+        finally:
+            if tf is not None:  # pragma: no cover
+                unlink(tf.name)
+                rmdir(idir)
+                rmdir(tdir)
+            c.close()
+            db.close()
+            s.close()
 
-        # In the leaf node we see entries for the parent and grandparent
-        data = marshal.loads(om.sub.subsub.manage_FTPlist(req))
-        self.assertEqual(len(data), 2)
-        self.assertEqual(data[0][0], '.')
-        self.assertEqual(data[1][0], '..')
+
+class _CallResult:
+    """Auxiliary class to provide defined call results."""
+    def __init__(self, result):
+        self.result = result
+
+    def __call__(self):
+        return self.result
 
 
 _marker = object()
@@ -611,29 +652,16 @@ class TestCheckValidId(unittest.TestCase):
         self.assertEqual(str(e),
                          "('Empty or invalid id specified', '')")
 
-    @unittest.skipUnless(PY2, 'Python 3 cannot handle bytestrings here.')
-    def test_unicode_py2(self):
-        e = self.assertBadRequest(u'abc☃')
-        self.assertEqual(
-            str(e), "('Empty or invalid id specified', u'abc\\u2603')")
+    def test_unicode(self):
+        self._callFUT(self._makeContainer(), 'abc☃')
 
-    @unittest.skipIf(PY2, 'Python 2 cannot handle unicode / text here.')
-    def test_unicode_py3(self):
-        self._callFUT(self._makeContainer(), u'abc☃')
-
-    @unittest.skipUnless(PY2, 'Python 3 cannot handle bytestrings here.')
-    def test_encoded_unicode_py2(self):
-        self._callFUT(self._makeContainer(), u'abcö'.encode('utf-8'))
-
-    @unittest.skipIf(PY2, 'Python 2 cannot handle unicode / text here.')
-    def test_encoded_unicode_py3(self):
-        e = self.assertBadRequest(u'abcö'.encode('utf-8'))
+    def test_encoded_unicode(self):
+        e = self.assertBadRequest('abcö'.encode())
         self.assertEqual(str(e),
                          "('Empty or invalid id specified', "
                          "b'abc\\xc3\\xb6')")
 
-    @unittest.skipUnless(PY2, 'Python 3 cannot handle bytestrings here.')
-    def test_unprintable_characters_py2(self):
+    def test_unprintable_characters(self):
         # We do not allow the first 31 ASCII characters. \x00-\x19
         # We do not allow the DEL character. \x7f
         e = self.assertBadRequest('abc\x10')
@@ -645,19 +673,6 @@ class TestCheckValidId(unittest.TestCase):
                          'The id "abc\x7f" contains characters illegal'
                          ' in URLs.')
 
-    @unittest.skipIf(PY2, 'Python 2 cannot handle unicode / text here.')
-    def test_unprintable_characters_py3(self):
-        # We do not allow the first 31 ASCII characters. \x00-\x19
-        # We do not allow the DEL character. \x7f
-        e = self.assertBadRequest(u'abc\x10')
-        self.assertEqual(str(e),
-                         'The id "abc\x10" contains characters illegal'
-                         ' in URLs.')
-        e = self.assertBadRequest(u'abc\x7f')
-        self.assertEqual(str(e),
-                         'The id "abc\x7f" contains characters illegal'
-                         ' in URLs.')
-
     def test_fail_on_brackets_and_ampersand(self):
         # We do not allow this characters as they result in TaintedString (for
         # < and >) which are not allowed in hasattr.
