@@ -1,14 +1,9 @@
-# -*- coding: utf-8 -*-
-
 import unittest
 import warnings
 
-from six import text_type
-
 from AccessControl import safe_builtins
 from zExceptions import NotFound
 from zope.component.testing import PlacelessSetup
-from zope.location.interfaces import LocationError
 
 
 class EngineTestsBase(PlacelessSetup):
@@ -28,13 +23,13 @@ class EngineTestsBase(PlacelessSetup):
 
     def _makeContext(self, bindings=None):
 
-        class Dummy(object):
+        class Dummy:
             __allow_access_to_unprotected_subobjects__ = 1
 
             def __call__(self):
                 return 'dummy'
 
-        class DummyDocumentTemplate(object):
+        class DummyDocumentTemplate:
             __allow_access_to_unprotected_subobjects__ = 1
             isDocTemp = True
 
@@ -93,7 +88,7 @@ class EngineTestsBase(PlacelessSetup):
         self.assertEqual(ec.evaluate('dummy'), 'dummy')
 
     def test_evaluate_with_unimplemented_call(self):
-        class Dummy(object):
+        class Dummy:
             def __call__(self):
                 raise NotImplementedError()
 
@@ -102,6 +97,7 @@ class EngineTestsBase(PlacelessSetup):
         self.assertIs(ec.evaluate('dummy'), dummy)
 
     def test_evaluate_with_render_DTML_template(self):
+        # http://www.zope.org/Collectors/Zope/2232
         # DTML templates could not be called from a Page Template
         # due to an ImportError
         ec = self._makeContext()
@@ -184,15 +180,15 @@ class EngineTestsBase(PlacelessSetup):
         ec = self._makeContext()
         # XXX: can't do ec.evaluate(u'string:x') directly because ZopeContext
         # only bothers compiling true strings, not unicode strings
-        result = ec.evaluate(eng.compile(u'string:x'))
-        self.assertEqual(result, u'x')
-        self.assertIsInstance(result, text_type)
+        result = ec.evaluate(eng.compile('string:x'))
+        self.assertEqual(result, 'x')
+        self.assertIsInstance(result, str)
 
     def test_mixed(self):
         # 8-bit strings in unicode string expressions cause UnicodeDecodeErrors
         eng = self._makeEngine()
         ec = self._makeContext()
-        expr = eng.compile(u'string:$eightbit')
+        expr = eng.compile('string:$eightbit')
         self.assertRaises(UnicodeDecodeError,
                           ec.evaluate, expr)
         # But registering an appropriate IUnicodeEncodingConflictResolver
@@ -204,7 +200,7 @@ class EngineTestsBase(PlacelessSetup):
         from zope.component import provideUtility
         provideUtility(StrictUnicodeEncodingConflictResolver,
                        IUnicodeEncodingConflictResolver)
-        self.assertEqual(ec.evaluate(expr), u'äüö')
+        self.assertEqual(ec.evaluate(expr), 'äüö')
 
     def test_builtin_in_path_expr(self):
         ec = self._makeContext()
@@ -237,10 +233,10 @@ class UntrustedEngineTests(EngineTestsBase, unittest.TestCase):
         with self.assertRaises(NotFound):
             ec.evaluate("context/__class__")
 
-        with self.assertRaises((NotFound, LocationError)):
+        with self.assertRaises(NotFound):
             ec.evaluate("nocall: random/_itertools/repeat")
 
-        with self.assertRaises((NotFound, LocationError)):
+        with self.assertRaises(NotFound):
             ec.evaluate("random/_itertools/repeat/foobar")
 
 
@@ -286,7 +282,7 @@ class UnicodeEncodingConflictResolverTests(PlacelessSetup, unittest.TestCase):
         provideUtility(StrictUnicodeEncodingConflictResolver,
                        IUnicodeEncodingConflictResolver)
         resolver = getUtility(IUnicodeEncodingConflictResolver)
-        text = u'\xe4\xfc\xe4'
+        text = '\xe4\xfc\xe4'
         self.assertEqual(resolver.resolve(None, text, None), text)
 
     def testIgnoringResolver(self):
@@ -312,7 +308,7 @@ class UnicodeEncodingConflictResolverTests(PlacelessSetup, unittest.TestCase):
                        IUnicodeEncodingConflictResolver)
         resolver = getUtility(IUnicodeEncodingConflictResolver)
         self.assertEqual(resolver.resolve(None, b'\xe4\xfc\xf6', None),
-                         u'\ufffd\ufffd\ufffd')
+                         '\ufffd\ufffd\ufffd')
 
 
 class ZopeContextTests(unittest.TestCase):
@@ -329,7 +325,7 @@ class ZopeContextTests(unittest.TestCase):
         return self._getTargetClass()(engine, contexts)
 
     def _makeEngine(self):
-        class DummyEngine(object):
+        class DummyEngine:
             pass
         return DummyEngine()
 
